﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.IO;
using System.Windows.Forms;
using System.Globalization;
using System.Text.RegularExpressions;
using Microsoft.WindowsAPICodePack;
using Microsoft.WindowsAPICodePack.Dialogs;
using dotsplit.Properties;

namespace dotsplit
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        private bool IsVistaCompatible = false;

        public Window1()
        {
            InitializeComponent();
        }

        private void OnLoaded(object sender, RoutedEventArgs e)
        {
            OperatingSystem os = Environment.OSVersion;

            // Vista Lib:
            // http://adventuresinwpf.blogspot.com/2008/09/vistabridge-considered-harmful.html
            // if (os.Version.Major >= 6 && VistaLib.TaskDialog.GotComctl6())
            // ------------------------------------------
            // Zmena knihovny z VistaLib na Windows® API Code Pack for Microsoft® .NET Framework
            // http://code.msdn.microsoft.com/WindowsAPICodePack
            // 
            // V knihovne byla zmenena struktura IconUnion: http://code.msdn.microsoft.com/WindowsAPICodePack/WorkItem/View.aspx?WorkItemId=108
            if (os.Version.Major >= 6)
            {
                IsVistaCompatible = true;
            }

            // Naplneni comboboxu ulozenymi hodnotami velikosti casti
            // a vyber naposledy vybraneho zaznamu.
            FillComboBox();

            if (Settings.Default.SelectedSizeIndex < Settings.Default.PartSizes.Count
                && Settings.Default.SelectedSizeIndex > -1)
                PartSize.SelectedIndex = Settings.Default.SelectedSizeIndex;
        }

        public void OnBrowseFileClick(object sender, RoutedEventArgs e)
        {
            // Vyber zdrojoveho souboru.
            using (OpenFileDialog openDlg = new OpenFileDialog())
            {
                openDlg.Filter = "Všechny soubory|*.*";
                openDlg.FilterIndex = 1;
                openDlg.RestoreDirectory = true;

                if (openDlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    SourceFile.Text = openDlg.FileName;
                }
            }
        }

        public void OnBrowseDirectoryClick(object sender, RoutedEventArgs e)
        {
            using (FolderBrowserDialog folderDialog = new FolderBrowserDialog())
            {
                // Otevreni dialogu pro vyber adresare v posledne vybranem,
                // pokud existuje. V opacnem pripade se ulozena hodnota vymaze.
                if (Settings.Default.LastOutputDirectory.Trim().Length > 0
                    && System.IO.Directory.Exists(Settings.Default.LastOutputDirectory.Trim()))
                    folderDialog.SelectedPath = Settings.Default.LastOutputDirectory.Trim();
                else
                    Settings.Default.LastOutputDirectory = String.Empty;

                // Zobrazeni dialogu pro vyber adresare.
                if (folderDialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    DestinationDirectory.Text = folderDialog.SelectedPath;
                    Settings.Default.LastOutputDirectory = folderDialog.SelectedPath;
                }
            }
        }

        private void SourceFilePreviewDrag(object sender, System.Windows.DragEventArgs e)
        {
            // Povoleni zacatku Drag&Drop, pokud je pretahovan 1 soubor.
            if (e.Data.GetDataPresent(System.Windows.DataFormats.FileDrop))
            {
                string[] droppedFiles = e.Data.GetData(System.Windows.DataFormats.FileDrop, true) as string[];
                if (droppedFiles.Length == 1 && File.Exists(droppedFiles[0]))
                {
                    e.Effects = System.Windows.DragDropEffects.Copy;
                    e.Handled = true;
                }
            }
        }

        private void SourceFileDrop(object sender, System.Windows.DragEventArgs e)
        {
            // Povoleni dokonceni Drag&Drop, pokud je pretahovan 1 soubor.
            if (e.Data.GetDataPresent(System.Windows.DataFormats.FileDrop))
            {
                string[] droppedFiles = e.Data.GetData(System.Windows.DataFormats.FileDrop, true) as string[];
                if (droppedFiles.Length == 1 && File.Exists(droppedFiles[0]))
                {
                    SourceFile.Text = droppedFiles[0];
                }
            }
        }

        public void OnSplitFileClick(object sender, RoutedEventArgs e)
        {
            // Kontrola, jestli je zadano jmeno zdrojoveho souboru.
            if (SourceFile.Text.Trim().Length == 0)
            {
                System.Windows.MessageBox.Show("Zadejte jméno zdrojového souboru.", Title, MessageBoxButton.OK, MessageBoxImage.Warning);
                return;
            }

            // Kontrola jestli soubor existuje.
            if (!File.Exists(SourceFile.Text.Trim()))
            {
                System.Windows.MessageBox.Show("Zadejte jméno platného souboru.", Title, MessageBoxButton.OK, MessageBoxImage.Warning);
                return;
            }

            // Kontrola, jestli existuje cilovy adresar.
            if (DestinationDirectory.Text.Trim().Length > 0 && !Directory.Exists(DestinationDirectory.Text.Trim()))
            {
                System.Windows.MessageBox.Show("Zadejte jméno platného adresáře.", Title, MessageBoxButton.OK, MessageBoxImage.Warning);
                return;
            }

            // Prevod zadane velikosti na hodnotu v bytech.
            string combo_file_size, combo_multiplier;
            long? part_size = GetPartSize(PartSize.Text, out combo_file_size, out combo_multiplier);

            // Byla zadana hodnota odpovidajici vzoru ve funkci GetPartSize.
            if (part_size.HasValue)
            {
                // Kontrola, jestli je zadana velikost alespon 200 kB.
                if (part_size.Value < 204800)
                {
                    System.Windows.MessageBox.Show("Nejmenší povolená velikost části je 200 kB.", Title, MessageBoxButton.OK, MessageBoxImage.Warning);
                    return;
                }

                try
                {
                    // Slozeni textu pro rozbalovaci pole comboboxu.
                    string combo_value = Convert.ToDouble(combo_file_size).ToString();
                    if (combo_multiplier.Length > 0)
                        combo_value += " " + combo_multiplier;

                    // Pridani hodnoty do comboboxu, pokud ji jeste neobsahuje.
                    if (!Settings.Default.PartSizes.Contains(combo_value))
                    {
                        // Ulozeni hodnoty do konfiguracniho
                        // souboru aplikace.
                        Settings.Default.PartSizes.Add(combo_value);

                        // Smazani hodnot v comboboxu a nasledne vlozeni
                        // ulozenych hodnot serazenych podle velikosti souboru.
                        FillComboBox();

                        // Vyhledani noveho zaznamu v comboboxu.
                        for (int pos = 0; pos < PartSize.Items.Count; pos++)
                        {
                            if (PartSize.Items[pos].ToString() == combo_value)
                            {
                                PartSize.SelectedIndex = pos;
                                Settings.Default.SelectedSizeIndex = pos;
                                break;
                            }
                        }
                    }
                }
                catch (FormatException)
                {
                }

                // Spusteni procesu rozdeleni souboru ...
                string dest_dir;
                if (DestinationDirectory.Text.Trim().Length == 0)
                {
                    dest_dir = System.IO.Path.GetDirectoryName(SourceFile.Text.Trim());
                }
                else
                {
                    dest_dir = DestinationDirectory.Text.Trim();
                }
                
                Split(SourceFile.Text.Trim(), dest_dir, GetPartSize(PartSize.Text));
            }
            else
            {
                System.Windows.MessageBox.Show("Zadejte velikost výsledných souborů ve formátu:" + Environment.NewLine + "reálné číslo [kB|MB|GB].", Title, MessageBoxButton.OK, MessageBoxImage.Warning);
                PartSize.Focus();
                return;
            }
        }

        /// <summary>
        /// Prevede textovou hodnotu velikosti souboru na byty.
        /// </summary>
        /// <param name="SizeValue">Textova reprezentace velikosti souboru.</param>
        /// <returns>Velikost v bytech nebo null, pokud nebyl text rozpoznan.</returns>
        private long? GetPartSize(string SizeValue)
        {
            string combo_file_size, combo_multiplier;
            return GetPartSize(SizeValue, out combo_file_size, out combo_multiplier);
        }

        /// <summary>
        /// Prevede textovou hodnotu velikosti souboru na byty.
        /// </summary>
        /// <param name="SizeValue">Textova reprezentace velikosti souboru.</param>
        /// <param name="FileSize">Cast s cislicemi.</param>
        /// <param name="Multiplier">Hodnota nasobitele [kB, MB, GB].</param>
        /// <returns>Velikost v bytech nebo null, pokud nebyl text rozpoznan.</returns>
        private long? GetPartSize(string SizeValue, out string FileSize, out string Multiplier)
        {
            FileSize = Multiplier = String.Empty;

            // Nacteni aktualniho regionalniho nastaveni, pro ziskani desetinneho oddelovace.
            NumberFormatInfo nfi = NumberFormatInfo.CurrentInfo;

            // Vzor pro ziskani velikosti souboru a jednotek.
            string pattern = @"^(?<size>\d+(\" + nfi.NumberDecimalSeparator + @"\d*)?)\s*(?<multiplier>(kb|mb|gb)?)$";

            // Ziskani velikosti souboru z ComboBoxu PartSize.
            Regex reg = new Regex(pattern, RegexOptions.IgnoreCase);
            Match m = reg.Match(SizeValue);

            if (m.Success)
            {
                //System.Windows.MessageBox.Show("=" + m.Groups["size"].Value + "=");
                //System.Windows.MessageBox.Show("=" + m.Groups["multiplier"].Value + "=");

                try
                {
                    double part_size = Convert.ToDouble(m.Groups["size"].Value);
                    FileSize = m.Groups["size"].Value;

                    switch (m.Groups["multiplier"].Value.ToLower())
                    {
                        case "gb":
                            part_size *= 1024;
                            if (Multiplier.Length == 0)
                                Multiplier = "GB";
                            goto case "mb";
                        case "mb":
                            part_size *= 1024;
                            if (Multiplier.Length == 0)
                                Multiplier = "MB";
                            goto case "kb";
                        case "kb":
                            part_size *= 1024;
                            if (Multiplier.Length == 0)
                                Multiplier = "kB";
                            break;
                    }

                    return Convert.ToInt64(part_size);
                }
                catch (Exception e)
                {
                    // Tohle by nikdy nemelo nastat.
                    // Spravny format cisla by mel zachytit Regex.
                    System.Windows.MessageBox.Show(e.Message);
                }
            }

            return null;
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            // Ulozeni konfigurace aplikace.
            if (PartSize.SelectedIndex > -1)
                Settings.Default.SelectedSizeIndex = PartSize.SelectedIndex;

            Settings.Default.Save();
        }

        /// <summary>
        /// Odstrani hodnotu z ComboBoxu PartSize.
        /// </summary>
        /// <param name="sender">Button. Vlastnost Tag obsahuje textovou hodnotu, ktera bude z ComboBoxu odstranena.</param>
        private void OnComboRemoveClick(object sender, RoutedEventArgs e)
        {
            // Vyhledani zaznamu v comboboxu a jeho smazani.
            string combo_value = (sender as System.Windows.Controls.Button).Tag.ToString();
            if (combo_value.Length == 0)
                return;

            for (int pos = 0; pos < PartSize.Items.Count; pos++)
            {
                if (PartSize.Items[pos].ToString() == combo_value)
                {
                    PartSize.Items.RemoveAt(pos);
                    Settings.Default.PartSizes.Remove(combo_value);
                    break;
                }
            }
        }

        /// <summary>
        /// Seradi hodnoty z kolekce Settings.Default.PartSizes metodou GetPartSize a hodnotami naplni ComboBox PartSize.
        /// </summary>
        private void FillComboBox()
        {
            // Naplni combobox hodnotami z Settings.Default.PartSizes
            // a serazenych podle velikosti v bytech.
            PartSize.Items.Clear();

            foreach (string s in Settings.Default.PartSizes.Cast<string>().OrderBy(st => GetPartSize(st).GetValueOrDefault()))
                PartSize.Items.Add(s);
        }

        /// <summary>
        /// Provede kontrolu nezbytnych udaju a spusti rozdeleni souboru ve tride BackgroundWorker.
        /// </summary>
        /// <param name="SourceFile">Zdrojovy soubor.</param>
        /// <param name="DestinationDirectory">Cilovy adresar.</param>
        /// <param name="part_size">Pozadovana velikost casti.</param>
        private void Split(string SourceFile, string DestinationDirectory, long? part_size)
        {
            try
            {
                FileInfo fiSrc = new FileInfo(SourceFile);

                // Kontrola, jestli je soubor mensi, nez pozadovana
                // velikost casti souboru.
                if (fiSrc.Length <= part_size.GetValueOrDefault())
                {
                    System.Windows.MessageBox.Show("Soubor není větší, než požadovaná velikost.",
                        Title, MessageBoxButton.OK, MessageBoxImage.Warning);
                    return;
                }

                // Vypocet, kolik bude vytvoreno casti.
                double final_part_count = Math.Ceiling((double)fiSrc.Length / (double)part_size.GetValueOrDefault());
                if (final_part_count > 100)
                {
                    if (System.Windows.MessageBox.Show("Soubor bude rozdělen na " + final_part_count.ToString() + " částí. Chcete pokračovat?",
                        Title, MessageBoxButton.YesNo, MessageBoxImage.Question) != MessageBoxResult.Yes)
                        return;
                }

                // Kontrola, jestli se mohou prepsat pripadne jiz existujici soubory.
                if (!CheckFilesOverwrite(SourceFile, DestinationDirectory, (int)final_part_count))
                    return;

                // Parametry pro tridu BackgroundWorker.
                BackgroundWorkerParameters bwp = new BackgroundWorkerParameters();
                bwp.SourceFileInfo = fiSrc;
                bwp.DestinationDirectory = DestinationDirectory;
                bwp.PartSize = part_size.GetValueOrDefault();

                // Inicializace tridy BackgroundWorker.
                System.ComponentModel.BackgroundWorker bw = new System.ComponentModel.BackgroundWorker();
                bw.WorkerReportsProgress = true;

                // Konec procesu.
                bw.RunWorkerCompleted += (obj, args) =>
                {
                    SplitFile.IsEnabled = true;

                    if (args.Error != null)
                    {
                        System.Windows.MessageBox.Show(this, args.Error.Message, Title,
                            MessageBoxButton.OK, MessageBoxImage.Error);
                    }
                    else
                    {
                        if (args.Result != null)
                        {
                            BackgroundWorkerResult bw_result = args.Result as BackgroundWorkerResult;
                            DisplaySplitResult(bw_result.SourceFile,
                                bw_result.DestinationDirectory,
                                bw_result.CreatedParts,
                                bw_result.CmdFile);
                        }
                    }
                };

                // Aktualizace procent.
                bw.ProgressChanged += (obj, args) =>
                {
                    if (SplitProgress.Value != args.ProgressPercentage)
                    {
                        SplitProgress.Value = args.ProgressPercentage;
                    }
                };

                // Cely kod v nasledujicim bloku je spusten na pozadi.
                bw.DoWork += (obj, args) =>
                {
                    BackgroundWorkerParameters bw_arg = args.Argument as BackgroundWorkerParameters;
                    System.ComponentModel.BackgroundWorker bw_obj = obj as System.ComponentModel.BackgroundWorker;
                    bw_obj.ReportProgress(0);

                    int part_number = 0;
                    string file_base_name = System.IO.Path.GetFileNameWithoutExtension(bw_arg.SourceFileInfo.FullName);
                    List<string> created_parts = new List<string>();

                    // Zdrojovy a cilove soubory.
                    FileStream fs_in = null;
                    FileStream fs_out = null;

                    try
                    {
                        // Otevreni souboru pro cteni.
                        fs_in = bw_arg.SourceFileInfo.OpenRead();

                        // Vytvoreni 1. casti.
                        string part_file_name = String.Format("{0}.{1:000}", file_base_name, part_number++);
                        created_parts.Add(part_file_name);
                        fs_out = new FileStream(System.IO.Path.Combine(bw_arg.DestinationDirectory, part_file_name),
                            FileMode.Create, FileAccess.Write);

                        // Buffer pro data.
                        const int buffer_size = 200 * 1024;
                        byte[] buffer = new byte[buffer_size];

                        long readed = 0;
                        while (readed < fs_in.Length)
                        {
                            int data_length = fs_in.Read(buffer, 0, buffer_size);

                            if (fs_out.Length + (long)data_length > bw_arg.PartSize)
                            {
                                int buffer_offset = (int)(bw_arg.PartSize - fs_out.Length);
                                fs_out.Write(buffer, 0, buffer_offset);

                                // Uzavreni aktualni casti.
                                fs_out.Close();

                                // Vytvoreni nove.
                                part_file_name = String.Format("{0}.{1:000}", file_base_name, part_number++);
                                created_parts.Add(part_file_name);
                                fs_out = new FileStream(System.IO.Path.Combine(bw_arg.DestinationDirectory, part_file_name),
                                    FileMode.Create, FileAccess.Write);

                                // Ulozeni zbytku dat z bufferu.
                                fs_out.Write(buffer, buffer_offset, data_length - buffer_offset);
                            }
                            else
                            {
                                fs_out.Write(buffer, 0, data_length);
                            }

                            readed += (long)data_length;

                            // Aktualizace procent.
                            int percent_done = (int)(readed * 100 / fs_in.Length);
                            bw_obj.ReportProgress(percent_done);
                        }
                        
                        // Vytvoreni cmd souboru pro spojeni casti do puvodniho souboru.
                        part_file_name = String.Format("{0}.cmd", file_base_name);

                        // Znakova sada konzole.
                        int oem_cp = CultureInfo.CurrentCulture.TextInfo.OEMCodePage;

                        using (StreamWriter sw = new StreamWriter(System.IO.Path.Combine(bw_arg.DestinationDirectory, part_file_name),
                            false, System.Text.Encoding.GetEncoding(oem_cp /*852*/)))
                        {
                            sw.WriteLine("@echo off");
                            sw.WriteLine("echo Obnovení souboru '" + bw_arg.SourceFileInfo.Name + "'.");
                            sw.WriteLine("echo Pro úspěšné obnovení musí být v adresáři '%cd%' soubory:");
                            sw.WriteLine("echo '" + created_parts[0] + "'");
                            sw.WriteLine("echo ...");
                            sw.WriteLine("echo '" + created_parts[created_parts.Count - 1] + "'");
                            sw.WriteLine("echo !! Pokud je ve stejném adresáři soubor '" + bw_arg.SourceFileInfo.Name + "' bude přepsán. !!");
                            sw.WriteLine("echo Stiskněte Ctrl + C pro přerušení obnovy souboru nebo");
                            sw.WriteLine("pause");
                            sw.WriteLine("@echo on");

                            // Pomocny buffer pro casti v cmd souboru.
                            List<string> parts_buffer = new List<string>();
                            string cmd_line;

                            for (int i = 0; i < created_parts.Count; i++)
                            {
                                // Pridani aktualni casti do bufferu.
                                parts_buffer.Add(created_parts[i]);

                                // Spojeni do puvodniho souboru max. po 5 castech.
                                if (parts_buffer.Count == 5)
                                {
                                    // Ulozeni do cmd souboru.
                                    cmd_line = String.Format("copy /b {0} \"{1}\"",
                                        GetJoinedParts(parts_buffer),
                                        bw_arg.SourceFileInfo.Name);

                                    sw.WriteLine(cmd_line);

                                    // Smazani bufferu.
                                    parts_buffer.Clear();

                                    // Pokud zbyvaji dalsi casti k ulozeni,
                                    // do bufferu se na 1. misto ulozi jmeno orig. souboru.
                                    if (i < created_parts.Count - 1)
                                        parts_buffer.Add(bw_arg.SourceFileInfo.Name);
                                }
                            }

                            // Ulozeni zbytku casti.
                            if (parts_buffer.Count > 0)
                            {
                                cmd_line = String.Format("copy /b {0} \"{1}\"",
                                    GetJoinedParts(parts_buffer),
                                    bw_arg.SourceFileInfo.Name);

                                sw.WriteLine(cmd_line);
                            }
                        }

                        // Aktualizace bargrafu na 0%.
                        bw_obj.ReportProgress(0);

                        // Predani vysledku hlavnimu procesu.
                        BackgroundWorkerResult bw_result = new BackgroundWorkerResult(bw_arg.SourceFileInfo.Name,
                            bw_arg.DestinationDirectory,
                            created_parts,
                            part_file_name);

                        args.Result = bw_result;
                    }
                    catch (Exception)
                    {
                        // Preposlani zachycene vyjimky.
                        throw;
                    }
                    finally
                    {
                        // Uzavreni zdrojoveho souboru a aktualni casti.
                        if (fs_in != null)
                            fs_in.Close();

                        if (fs_out != null)
                            fs_out.Close();
                    }
                };

                // ----------------------------------------------
                // Spusteni noveho procesu pro rozdeleni souboru.
                SplitFile.IsEnabled = false;
                bw.RunWorkerAsync(bwp);
            }
            catch (Exception e)
            {
                SplitFile.IsEnabled = true;
                System.Windows.MessageBox.Show(e.Message, Title, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        /// <summary>
        /// Zobrazi souhrnou zpravu o rozdeleni souboru. V XP tridou MessageBox, ve Win Vista tridou TaskDialog.
        /// </summary>
        /// <param name="SourceFile">Zdrojovy soubor.</param>
        /// <param name="DestinationDirectory">Cilovy adresar.</param>
        /// <param name="CreatedParts">Seznam se jmeny vytvorenych casti.</param>
        /// <param name="CmdFile">Jmeno cmd souboru pro spojeni casti dohromady.</param>
        private void DisplaySplitResult(string SourceFile, string DestinationDirectory, List<string> CreatedParts, string CmdFile)
        {
            // Ve Win Vista zobrazi TaskDialog, v XP jen MessageBox.
            if (IsVistaCompatible)
            {
                using (new VistaLib.EnableThemingInScope(true))
                {
                    TaskDialog td = new TaskDialog();
                    td.Cancelable = true;
                    td.OwnerWindowHandle = new System.Windows.Interop.WindowInteropHelper(this).Handle;
                    td.Caption = Title;
                    td.DetailsExpanded = false;
                    td.Icon = TaskDialogStandardIcon.Information;

                    td.InstructionText = "Výsledek rozdělení souboru";
                    td.Text = "Soubor " + SourceFile + " byl rozdělen na " + CreatedParts.Count.ToString() + (CreatedParts.Count > 4 ? " částí." : " části.");

                    td.StandardButtons = TaskDialogStandardButtons.Close;
                    td.FooterText = "Spuštěním souboru " + CmdFile + " spojíte části dohromady.";
                    td.FooterIcon = TaskDialogStandardIcon.Information;

                    if (CreatedParts.Count < 100)
                    {
                        StringBuilder sb = new StringBuilder("Byly vytvořeny tyto soubory:" + Environment.NewLine);
                        foreach (string part in CreatedParts)
                        {
                            sb.Append(Environment.NewLine);
                            sb.Append(part);
                        }
                        sb.Append(Environment.NewLine + CmdFile);

                        td.DetailsExpandedText = sb.ToString();
                        td.DetailsExpandedLabel = "Skrýt detaily";
                        td.DetailsCollapsedLabel = "Zobrazit detaily";
                    }

                    TaskDialogCommandLink openFolderButton = new TaskDialogCommandLink("openFolderButton", "Otevřít složku s částmi souboru");
                    openFolderButton.Click += (cb_o, cb_args) =>
                    {
                        if (Directory.Exists(DestinationDirectory))
                            System.Diagnostics.Process.Start(DestinationDirectory);
                    };

                    td.Controls.Add(openFolderButton);
                    
                    td.Show();
                }
            }
            else
            {
                StringBuilder sb = new StringBuilder();
                sb.Append("Soubor " + SourceFile + " byl rozdělen na " + CreatedParts.Count.ToString() + (CreatedParts.Count > 4 ? " částí." : " části."));
                sb.Append(Environment.NewLine);
                sb.Append(Environment.NewLine);
                sb.Append("Spuštěním souboru " + CmdFile + " spojíte části dohromady.");

                System.Windows.MessageBox.Show(sb.ToString(), Title, MessageBoxButton.OK, MessageBoxImage.Information);
            }
        }

        /// <summary>
        /// Zkontroluje, jestli v cilovem adresari existuji soubory, ktere aplikace vytvori. Pokud existuji, nabidne uzivateli moznost je prepsat.
        /// </summary>
        /// <param name="FileName">Jmeno zdrojoveho souboru.</param>
        /// <param name="TargetDirectory">Cilovy adresar.</param>
        /// <param name="FileParts">Pocet casti, ktere aplikace vytvori.</param>
        private bool CheckFilesOverwrite(string FileName, string TargetDirectory, int FileParts)
        {
            string file_base_name = System.IO.Path.GetFileNameWithoutExtension(FileName);

            for (int i = 0; i < FileParts; i++)
            {
                string part_file_name = String.Format("{0}.{1:000}", file_base_name, i);
                part_file_name = System.IO.Path.Combine(TargetDirectory, part_file_name);

                if (File.Exists(part_file_name))
                {
                    OverwriteFile ovf = new OverwriteFile();
                    ovf.Owner = this;
                    ovf.FileName = part_file_name;

                    bool? dlg_result = ovf.ShowDialog();

                    if (!dlg_result.GetValueOrDefault())
                        return false;
                    else if (ovf.OverwriteAllFiles)
                        return true;
                }
            }

            return true;
        }

        private string GetJoinedParts(List<string> Parts)
        {
            return String.Join("+", Parts
                .Select(p => "\"" + p + "\"")
                .ToArray());
        }
    }
}
